import glob
import json
import os
from pathlib import Path

import click
import torch
import torch.nn as nn
import torch.nn.functional as f
import torch.optim as optim
import wandb
from dotenv import find_dotenv, load_dotenv
from numpy import mean
from sklearn.model_selection import KFold
from torch.nn import NLLLoss

from contextual_ner.models.classifiers import classifier_factory
from contextual_ner.models.utils import (
    init_tokenizer,
    save_model,
    monitor_output_example,
    monitor_model_eval,
    init_model,
    logger,
    log_classification_report,
    create_dataloader,
    empty_gpu,
)


# pylint: disable=too-many-arguments, too-many-locals


@click.command()
@click.option("--learning-rate", required=True)
@click.option("--classifier-name", required=True)
@click.option("--batch-size", required=True)
@click.option("--num-epochs", required=True)
@click.option("--folds", required=True)
@click.option("--model-name", required=True)
@click.option("--model-path", required=True)
@click.option("--dataset-path", required=True)
def main(
    learning_rate: float,
    batch_size: int,
    num_epochs: int,
    folds: int,
    model_name: str,
    model_path: str,
    dataset_path: str,
    classifier_name: str,
):
    """
    Args:
        classifier_name:
        learning_rate: the model's learning rate
        batch_size: size of the mini-batch
        num_epochs: epoch's number during training
        folds: we use K-fold to evaluate the model, insert the number of folds here
        model_name: the huggingface model name
        model_path: the saving path for the trained model, we record k model, k is a number of folds
        dataset_path: the path of the dataset
    """
    hyperparameter_defaults = dict(
        lr=float(learning_rate),
        batch_size=int(batch_size),
        num_epochs=int(num_epochs),
        folds=int(folds),
        model_name=model_name,
        model_path=model_path,
    )
    # monitor the model with wandb
    wandb.init(config=hyperparameter_defaults)
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
    dataset = sorted(glob.glob(dataset_path + "/*.pickle"), key=os.path.getsize)
    metadata = json.load(open(glob.glob(dataset_path + "/*.json")[0], "r"))
    labels = metadata["labels"]
    num_labels = len(labels)
    logger.info("Using device: {}".format(device))
    logger.info("Using tokenizer from model : {}".format(model_name))
    tokenizer = init_tokenizer(hyperparameter_defaults["model_name"])
    criterion = nn.NLLLoss(
        weight=torch.Tensor([metadata["labels weights"][label] for label in labels]).to(
            device
        )
    )
    folds = KFold(n_splits=hyperparameter_defaults["folds"], shuffle=False)
    for id_fold, fold in enumerate(folds.split(dataset)):
        # for each fold with we iterate a training loop and a evaluation loop
        # we iterate a different evaluation and training batches, we compute the CV loss to
        # check the integrity of the corpus. We use small corpora to fine-tuning the model
        # with a small masking task to downstream to another task (NER, QA ...)
        # to a specialized domain.

        logger.info("beginning fold n°{}".format(id_fold + 1))
        model = init_model(hyperparameter_defaults["model_name"], device)
        classifier = classifier_factory(
            classifier_name,
            device,
            **{"zn_size": model.config.hidden_size, "num_labels": num_labels},
        )
        logger.info(f"using the following classifier configuration : {classifier}")
        parameters = list(model.parameters()) + list(classifier.parameters())
        optimizer = optim.Adam(parameters, lr=hyperparameter_defaults["lr"])
        eval_dataloader, train_dataloader = create_dataloader(
            dataset, fold, hyperparameter_defaults, tokenizer
        )
        wandb.watch(model)
        train_model(
            model,
            classifier,
            train_dataloader,
            optimizer,
            criterion,
            device,
            id_fold,
            hyperparameter_defaults["num_epochs"],
        )
        save_model(classifier, hyperparameter_defaults, id_fold, model, model_name)
        cv_loss, recall, precision, f1_score = eval_model(
            model, classifier, eval_dataloader, criterion, device, labels
        )
        # delete model to empty some space memory
        empty_gpu(model)

        # Monitor current fold
        wandb.log({"CV loss": cv_loss})
        wandb.log({"CV recall": recall})
        wandb.log({"CV precision": precision})
        wandb.log({"CV F1 score": f1_score})
        logger.info("🔎 evaluation is finished, here is the different metrics :")
        logger.info(f"cv recall : {recall}")
        logger.info(f"cv precision: {precision}")
        logger.info(f"cv F1 score: {f1_score}")
        logger.info(f"cv loss: {cv_loss}")


def compute_loss(
    predictions: torch.Tensor, targets: torch.Tensor, criterion: NLLLoss
) -> torch.Tensor:
    """
    this method is used to compute the loss of the model with the predicted tensor and
    the target tensor. We apply other operation to arrange tensor in order to select only
    the last hidden states of our model.
    Args:
        predictions: the predicted model vector
        targets: the targeted vector, it is a vector with just the masked tokens
        criterion: the loss function

    Returns:
        the loss score
    """
    rearranged_output = predictions.contiguous().view(
        predictions.shape[0] * predictions.shape[1], -1
    )
    rearranged_target = targets.contiguous().view(-1)
    loss = criterion(rearranged_output, rearranged_target)
    return loss


def train_model(
    model, classifier, dataloader, optimizer, criterion, device, id_fold, epochs
):
    """
    the model training loop
    Args:
        classifier:
        model: the transformer model
        dataloader: the pytorch dataloader
        optimizer: the pytorch optimizer
        criterion: the pytorch loss function
        device: the pytorch device
        id_fold: the fold id process in the main loop
        epochs: the number of epochs
    """
    model.train()
    classifier.train()
    epoch_loss = []
    logger.info("🎓 model training step is started")
    for epoch in range(epochs):
        logger.info(f"processing epochs {epoch + 1}/{epochs}")
        for batch in dataloader:
            optimizer.zero_grad()
            predictions = classifier(
                model(
                    input_ids=batch["tokens"].to(device),
                    attention_mask=batch["attention_mask"].to(device),
                ).last_hidden_state
            )
            predictions_scores = f.log_softmax(predictions, dim=2)
            loss = compute_loss(
                predictions_scores, batch["target"].to(device), criterion
            )
            loss.backward()
            torch.nn.utils.clip_grad_norm_(model.parameters(), 1.0)
            optimizer.step()
            wandb.log({"loss {}".format(id_fold): loss.item()})
            epoch_loss.append(loss.item())
    logger.info("train epoch loss : {}".format(mean(epoch_loss)))
    logger.info("🎓 model training step is finished")


def eval_model(model, classifier, dataloader, criterion, device, labels):
    """
    the model evaluation loop
    Args:
        model: the transformer model
        dataloader: the pytorch dataloader
        criterion: the pytorch loss function
        device: the pytorch device
        classifier:
        labels:
    Returns:
        the mean epoch loss using to compute the CV score
    """
    model.eval()
    classifier.eval()
    epoch_loss = []
    evaluation_measure = {
        "recall": [],
        "precision": [],
        "F1 score": [],
        "pred_labels": [],
        "true_labels": [],
    }
    logger.info("model evaluation step is started")
    with torch.no_grad():
        batch = None
        for batch in dataloader:
            predictions = classifier(
                model(
                    input_ids=batch["tokens"].to(device),
                    attention_mask=batch["attention_mask"].to(device),
                ).last_hidden_state
            )
            predictions_scores = f.log_softmax(predictions, dim=2)
            loss = compute_loss(
                predictions_scores, batch["target"].to(device), criterion
            )
            epoch_loss.append(loss.item())
            precision, recall, f1_score = monitor_model_eval(
                predictions_scores.to("cpu"), batch
            )
            evaluation_measure["recall"].append(recall)
            evaluation_measure["precision"].append(precision)
            evaluation_measure["F1 score"].append(f1_score)
            evaluation_measure["true_labels"].append(batch["target"])
            evaluation_measure["pred_labels"].append(predictions_scores.to("cpu"))
    if batch is not None:
        monitor_output_example(batch, dataloader, labels, predictions_scores)
    log_classification_report(
        evaluation_measure["true_labels"], evaluation_measure["pred_labels"], labels
    )
    logger.info("model evaluation step is finished")
    return (
        mean(epoch_loss),
        mean(evaluation_measure["recall"]),
        mean(evaluation_measure["precision"]),
        mean(evaluation_measure["F1 score"]),
    )


if __name__ == "__main__":
    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]
    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())
    # pylint: disable=no-value-for-parameter
    main()



import logging
from logging.config import fileConfig
from os import path

import torch
from torch import nn

fileConfig(path.join(path.dirname(__file__), "../resources/log_config.ini"))
logger = logging.getLogger(__name__)


class CNN(nn.Module):
    def __init__(self, zn_size, num_labels):
        super().__init__()
        self.conv1d = nn.Conv1d(zn_size, num_labels, kernel_size=3, padding=1)

    def forward(self, z_n):
        return self.conv1d(z_n.transpose(1, 2)).transpose(2, 1)


class LinearClassifier(nn.Module):
    def __init__(self, zn_size, num_labels):
        super().__init__()
        self.linear = nn.Linear(zn_size, num_labels)

    def forward(self, z_n):
        return self.linear(z_n)


class GRU(nn.Module):
    def __init__(self, zn_size, num_labels):
        super().__init__()
        self.gru = nn.GRU(zn_size, 300, 2, bidirectional=True, batch_first=True)
        self.linear = nn.Linear(300 * 2, num_labels)

    def forward(self, z_n):
        gru_out, _ = self.gru(z_n)
        return self.linear(gru_out)


def classifier_factory(
    model_name: str, device: torch.device, **parameters
) -> nn.Module:
    # pylint: disable=no-else-return
    if model_name == "linear":
        return LinearClassifier(parameters["zn_size"], parameters["num_labels"]).to(
            device
        )
    elif model_name == "cnn":
        return CNN(parameters["zn_size"], parameters["num_labels"]).to(device)
    elif model_name == "gru":
        return GRU(parameters["zn_size"], parameters["num_labels"]).to(device)
    raise ValueError(logger.error(f"{model_name}: no such model exists !"))



import logging
import pickle
import json
import glob
from logging.config import fileConfig
from os import path
from pathlib import Path

import click
import pandas as pd
import numpy as np
from transformers import BatchEncoding
from dotenv import find_dotenv, load_dotenv
from tqdm import tqdm
from sklearn.utils.class_weight import compute_class_weight

from contextual_ner.models.utils import init_tokenizer

fileConfig(path.join(path.dirname(__file__), "../resources/log_config.ini"))
logger = logging.getLogger(__name__)


@click.command()
@click.option("--input-filepath", type=click.Path(), required=True)
@click.option("--output-filepath", type=click.Path(), required=True)
@click.option("--model-name", required=True)
def main(input_filepath: Path, output_filepath: Path, model_name: str):
    """Runs data processing scripts to turn raw data from (../raw) into
    cleaned data ready to be analyzed (saved in ../processed).
    """
    logger.info("making final data set from raw data")
    tokenizer = init_tokenizer(model_name)
    total_tokenized_labels = []
    with open(input_filepath, "rb") as file:
        quaero_dataset = pickle.load(file)
        corpus_labels = get_labels(quaero_dataset) + ["O"]
        quaero_dataset = sorted(
            quaero_dataset, key=lambda x: len(tokenizer.encode(x[0]))
        )
        for idx_example, example in enumerate(
            tqdm(quaero_dataset, position=1, leave=False, desc="pickle parsing")
        ):
            tokens = tokenizer(example[0], return_offsets_mapping=True)
            tokenized_labels = extract_tokenized_labels(tokens, example[1], "O")
            total_tokenized_labels += tokenized_labels
            id_labels = [corpus_labels.index(label) for label in tokenized_labels]
            data = {
                "sub_tokens_ids": tokens["input_ids"],
                "labels": tokenized_labels,
                "text": example[0],
                "sub_tokens": tokenizer.convert_ids_to_tokens(tokens["input_ids"])[
                    1:-1
                ],
                "labels_ids": id_labels,
            }
            write_file(output_filepath, idx_example, **data)
    weights = construct_weight_labels_dict(total_tokenized_labels, corpus_labels)
    write_metadata(output_filepath, corpus_labels, weights)


def write_file(output_filepath: Path, idx_example: int, **data: dict):
    normalize_index = str(idx_example).zfill(4)
    with open(f"{output_filepath}/quaero-{normalize_index}.pickle", "wb") as file:
        pickle.dump(data, file)
    with open(f"{output_filepath}/quaero-{normalize_index}.csv", "w") as file:
        csv = pd.DataFrame(columns=data.keys())
        for col, value in data.items():
            csv.at[0, col] = str(value) if isinstance(value, list) else value
        csv.to_csv(file)


def construct_weight_labels_dict(
    total_tokenized_labels: list, corpus_labels: list
) -> dict:
    labels_index = [corpus_labels.index(l) for l in corpus_labels]
    total_corpus_index = [corpus_labels.index(t) for t in total_tokenized_labels]
    weights = compute_class_weight(
        class_weight="balanced",
        classes=np.array(labels_index),
        y=np.array(total_corpus_index),
    )
    return {corpus_labels[idx]: weight for idx, weight in enumerate(weights)}


def write_metadata(output_filepath: Path, labels: list, weights: dict):
    files = glob.glob(f"{output_filepath}/*.pickle")
    len_files = len(files)
    len_labels = len(labels)
    with open(f"{output_filepath}/metadata.json", "w") as file:
        json.dump(
            {
                "files": files,
                "number of files": len_files,
                "labels": labels,
                "number of labels": len_labels,
                "labels weights": weights,
            },
            file,
        )


def get_labels(dataset: dict) -> list:
    entities = list(set(label[2] for el in dataset for label in el[1]["entities"]))
    entities = [[f"B-{entity}", f"I-{entity}"] for entity in entities]
    return [entity for ent_list in entities for entity in ent_list]


def extract_tokenized_labels(tokens: BatchEncoding, labels: dict, o_value: str) -> list:
    ids = tokens["input_ids"][1:-1]
    tokenized_labels = [o_value] * len(ids)
    offsets = tokens["offset_mapping"][1:-1]
    start_token = 0
    end_token = 0
    for label in list(labels.values())[0]:
        for idx_offset, offset in enumerate(offsets):
            if offset[0] == label[0]:
                start_token = idx_offset
            if offset[1] == label[1]:
                end_token = idx_offset
                break

        tokenized_labels[start_token : end_token + 1] = ["I-" + label[2]] * len(
            tokenized_labels[start_token : end_token + 1]
        )
        tokenized_labels[start_token] = "B-" + label[2]
    return [o_value] + tokenized_labels + [o_value]


if __name__ == "__main__":
    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]
    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())
    # pylint: disable=no-value-for-parameter
    main()



import pickle

import torch
from torch.nn.utils.rnn import pad_sequence
from torch.utils.data import Dataset
from transformers import CamembertTokenizerFast


class CamemBertNerDataset(Dataset):
    def __init__(self, dataset: list, tokenizer: CamembertTokenizerFast):
        self.tokenizer = tokenizer
        self.dataset = dataset
        self.pad_id = tokenizer.pad_token_id

    def __getitem__(self, index) -> dict:
        example = pickle.load(open(self.dataset[index], "rb"))
        return {
            "tokens": example["sub_tokens_ids"],
            "target": example["labels_ids"],
        }

    def __len__(self) -> int:
        return len(self.dataset)

    def masked_lm_collate(self, batch: list):
        tokens = self.pad_batch(batch, "tokens", self.pad_id)
        target = self.pad_batch(batch, "target", -100)
        attention_mask = (tokens != self.pad_id).int()

        return {"tokens": tokens, "attention_mask": attention_mask, "target": target}

    @staticmethod
    def pad_batch(batch, element_name, pad_id):
        return pad_sequence(
            [torch.LongTensor(example[element_name]) for example in batch],
            batch_first=True,
            padding_value=pad_id,
        )



from torch import nn


def aggregate_annotations(annotated_text):
    aggregate_labels = []
    start = 0
    end = 0
    current_label = ""
    for token in annotated_text["spans"]:
        if (
            token["label"].startswith("I")
            and current_label == token["label"].split("-")[-1]
        ):
            end = token["end"]
        elif token["label"].startswith("B-"):
            aggregate_labels.append(
                {"start": start, "end": end, "label": current_label}
            )
            start = token["start"]
            end = token["end"]
            current_label = token["label"].split("-")[-1]

    annotation = [start, end, current_label]
    if annotation != [
        aggregate_labels[-1]["start"],
        aggregate_labels[-1]["end"],
        aggregate_labels[-1]["label"],
    ]:
        aggregate_labels.append(
            {"start": annotation[0], "end": annotation[1], "label": annotation[-1]}
        )
    return {"text": annotated_text["text"], "labels": aggregate_labels[1:]}


class CamembertForTokenClassification(nn.Module):
    def __init__(self, camembert_model, classifier, tokenizer, labels):
        super().__init__()
        self.camembert_model = camembert_model
        self.classifier = classifier
        self.tokenizer = tokenizer
        self.labels = labels

    def forward(self, text):
        tokens = self.tokenizer(text, return_tensors="pt", return_offsets_mapping=True)
        logits = self.classifier(
            self.camembert_model(
                input_ids=tokens["input_ids"], attention_mask=tokens["attention_mask"]
            ).last_hidden_state
        )
        output_labels = [
            self.labels[i] for i in logits.squeeze(0).argmax(dim=1).tolist()
        ]
        annotated_text = {
            "text": text,
            "spans": [
                {"start": offset[0], "end": offset[1], "label": label}
                for offset, label in zip(
                    tokens["offset_mapping"].squeeze(0).tolist()[1:-1],
                    output_labels[1:-1],
                )
            ],
        }
        return aggregate_annotations(annotated_text)



from setuptools import find_packages, setup

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

print(find_packages())
setup(
    name="name-entity-extraction-for-contextual-embedding",
    packages=find_packages(),
    version="0.35.0",
    description="a machine learning project for mlm task for contextual embedding",
    author="Simon Meoni",
    license="MIT",
    install_requires=requirements,
    entry_points="""
    [console_scripts]
    camembert_ner_ft=contextual_ner.models.camembert_ner_ft:main
    make_dataset=contextual_ner.data.make_dataset:main
    """,
    package_data={
        "contextual_ner": [
            "resources/log_config.ini",
        ]
    },
)



import gc
import logging
import os
from datetime import datetime
from os import path
from logging.config import fileConfig
import numpy as np
import wandb
from sklearn.metrics import precision_recall_fscore_support, classification_report
from torch import nn
from torch.nn.utils.rnn import pad_sequence
from torch.utils.data import DataLoader
from transformers import AutoTokenizer, AutoModel
import torch

from contextual_ner.data.camembert_ner_dataset import CamemBertNerDataset
from contextual_ner.models.camembert_for_token_classification import (
    CamembertForTokenClassification,
)
from contextual_ner.models.classifiers import classifier_factory

fileConfig(path.join(path.dirname(__file__), "../resources/log_config.ini"))
logger = logging.getLogger("camembert-ner-ft")


def count_parameters(mdl):
    return sum(p.numel() for p in mdl.parameters() if p.requires_grad)


def init_tokenizer(model_name):
    logger.info("✂ Loading tokenizer...")
    tokenizer = AutoTokenizer.from_pretrained(model_name, use_fast=True)
    return tokenizer


def save_model(classifier, hyperparameter_defaults, id_fold, model, model_name):
    logger.info("💾 saving model ..")
    current_datetime = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
    save_location = hyperparameter_defaults["model_path"]
    model_name = f"{model_name}-" f"{current_datetime}-fold-{id_fold + 1}.bin"
    if not os.path.exists(save_location):
        os.makedirs(save_location)
    save_location = os.path.join(save_location, model_name)
    torch.save(
        CamembertForTokenClassification(model, classifier, None, None).state_dict(),
        save_location,
    )
    wandb.save(save_location)


def monitor_output_example(batch, dataloader, labels, predictions_scores):
    decode_ids = dataloader.dataset.tokenizer.convert_ids_to_tokens(batch["tokens"][-1])
    seq_token_index = decode_ids.index("</s>")
    decode_ids = decode_ids[1:seq_token_index]
    true_labels = [
        labels[label_id] for label_id in batch["target"][-1][1:seq_token_index]
    ]
    pred_labels = [
        labels[label_id]
        for label_id in predictions_scores[-1].argmax(dim=1)[1:seq_token_index]
    ]
    logger.info(
        "here is one example tagged by the model (text/true labels/pred labels):"
    )
    logger.info(decode_ids)
    logger.info(true_labels)
    logger.info(pred_labels)


def init_model(model_name, device):
    logger.info("🧠 Loading model...")
    model = AutoModel.from_pretrained(model_name)
    if device == "cuda":
        model = nn.DataParallel(model).to(device)
    model = model.to(device)
    logger.info(f"The model has {count_parameters(model):,} trainable parameters")
    return model


def monitor_model_eval(
    predictions_scores: torch.Tensor, batch: dict
) -> (float, float, float):
    y_true = batch["target"].contiguous().reshape(-1)
    y_pred = predictions_scores.contiguous().argmax(dim=2).reshape(-1)
    labels = pad_prediction(y_pred, y_true)
    precision, recall, f1_score, _ = precision_recall_fscore_support(
        y_true, y_pred, average="macro", labels=labels
    )
    return precision, recall, f1_score


def pad_prediction(y_pred, y_true):
    pad_idx = (y_true == -100).nonzero(as_tuple=False).flatten()
    y_pred[pad_idx] = -100
    labels = np.intersect1d(y_pred.unique(), y_true.unique())
    labels = np.delete(labels, np.where(labels == -100))
    return labels


def log_classification_report(true_labels: list, pred_labels: list, labels: list):
    flatten_true_labels = pad_sequence(
        [target for batch in true_labels for target in batch],
        batch_first=True,
        padding_value=-100,
    ).reshape(-1)
    flatten_pred_labels = (
        pad_sequence(
            [pred for batch in pred_labels for pred in batch],
            batch_first=True,
            padding_value=-100,
        )
        .argmax(dim=2)
        .reshape(-1)
    )
    labels_report_id = pad_prediction(flatten_true_labels, flatten_pred_labels)
    labels_report = [labels[label] for label in labels_report_id]
    report = classification_report(
        flatten_true_labels,
        flatten_pred_labels,
        labels=labels_report_id,
        target_names=labels_report,
    )
    logger.info(f"here is the detailed report of the evaluation : \n\n{report}")


def create_dataloader(dataset, fold, hyperparameter_defaults, tokenizer):
    train_fold, eval_fold = fold
    train_fold = [dataset[file_index] for file_index in train_fold]
    eval_fold = [dataset[file_index] for file_index in eval_fold]
    train_dataset = CamemBertNerDataset(train_fold, tokenizer)
    eval_dataset = CamemBertNerDataset(eval_fold, tokenizer)
    train_dataloader = DataLoader(
        dataset=train_dataset,
        batch_size=hyperparameter_defaults["batch_size"],
        shuffle=False,
        drop_last=True,
        collate_fn=train_dataset.masked_lm_collate,
    )
    eval_dataloader = DataLoader(
        dataset=eval_dataset,
        batch_size=hyperparameter_defaults["batch_size"],
        shuffle=False,
        drop_last=True,
        collate_fn=eval_dataset.masked_lm_collate,
    )
    return eval_dataloader, train_dataloader


def empty_gpu(model):
    del model
    gc.collect()
    torch.cuda.empty_cache()


def load_from_fine_tuned(device, state_dict_path, labels):
    logger.info("💾 loading a fine tune model ...")
    state_dict = torch.load(state_dict_path)
    if "classifier.linear.bias" in state_dict:
        classifier_name = "linear"
        num_labels = state_dict["classifier.linear.weight"].shape[0]
    elif "classifier.cnn.bias" in state_dict:
        classifier_name = "cnn"
        num_labels = state_dict["cnn.linear.weight"].shape[0]
    else:
        classifier_name = "gru"
        num_labels = state_dict["gru.linear.weight"].shape[0]
    zn_size = state_dict["camembert_model.pooler.dense.weight"].shape[0]
    camembert_model = CamembertForTokenClassification(
        init_model("camembert-base", device),
        classifier_factory(
            classifier_name, device, **{"zn_size": zn_size, "num_labels": num_labels}
        ),
        init_tokenizer("camembert-base"),
        labels,
    )
    camembert_model.load_state_dict(state_dict)
    camembert_model.eval()
    return camembert_model
